@@ -102,6 +102,8 @@ struct MultiWin {
     gboolean title_template_locked;
     gboolean clear_demands_attention;
     int best_tab_width;
+    GtkWidget *add_tab_button;
+    gboolean show_add_tab_button;
 };
 
 static double multi_win_zoom_factors[] = {
@@ -166,7 +168,7 @@ static void multi_win_restore_size(MultiWin *win)
 #if !GTK_CHECK_VERSION(3, 0, 0)
     GtkRequisition win_rq, term_rq;
 #endif
-    
+
     if (!tab || !tab->active_widget ||
             !gtk_widget_get_realized(tab->active_widget))
     {
@@ -205,7 +207,7 @@ void multi_tab_popup_menu(MultiTab * tab, guint button, guint32 event_time)
     GtkMenu *menu = GTK_MENU(menutree_get_top_level_widget
             (tab->parent->show_menu_bar ? tab->parent->short_popup :
             tab->parent->popup_menu));
-    
+
     gtk_menu_set_screen(menu, gtk_widget_get_screen(tab->widget));
     gtk_menu_popup(menu, NULL, NULL, NULL, NULL, button, event_time);
 }
@@ -242,7 +244,7 @@ void multi_tab_init(MultiTabFiller filler, MultiTabDestructor destructor,
 MultiTab *multi_tab_new(MultiWin * parent, gpointer user_data_template)
 {
     MultiTab *tab = g_new0(MultiTab, 1);
-    int pos;
+    guint pos;
 
     tab->parent = parent;
     tab->status_stock = NULL;
@@ -366,7 +368,7 @@ static void multi_win_set_geometry_hints_for_tab(MultiWin * win, MultiTab * tab)
 static char *make_title(const char *template, const char *title)
 {
     char *title0 = NULL;
-    
+
     if (template)
     {
         if (strstr(template, "%s"))
@@ -398,7 +400,7 @@ static gboolean check_title_template(const char *tt)
     const char *c;
     gboolean lwpc = FALSE;
     gboolean got_pcs = FALSE;
-    
+
     c = tt;
     do
     {
@@ -437,7 +439,7 @@ static gboolean validate_title_template(GtkWindow *parent, const char *tt)
     if (tt && !check_title_template(tt))
     {
         static char *bad_template = NULL;
-    
+
         if (!bad_template || strcmp(bad_template, tt))
         {
             dlg_warning(parent,
@@ -501,7 +503,7 @@ void multi_tab_set_window_title(MultiTab * tab, const char *title)
 void multi_tab_set_window_title_template(MultiTab * tab, const char *template)
 {
     GtkWidget *gwin = tab->parent ? tab->parent->gtkwin : NULL;
-    
+
     if (!validate_title_template(gwin ? GTK_WINDOW(gwin) : NULL,
             template))
     {
@@ -607,7 +609,7 @@ static void multi_win_shade_for_previous_tab(MultiWin *win)
 {
     gboolean shade = multi_win_override_shade_next_prev_tab(win) &&
             multi_win_at_first_tab(win);
-            
+
     multi_win_shade_item_in_both_menus(win, MENUTREE_TABS_PREVIOUS_TAB,
             shade);
     multi_win_shade_item_in_both_menus(win, MENUTREE_TABS_MOVE_TAB_LEFT,
@@ -618,7 +620,7 @@ static void multi_win_shade_for_next_tab(MultiWin * win)
 {
     gboolean shade = multi_win_override_shade_next_prev_tab(win) &&
             multi_win_at_last_tab(win);
-            
+
     multi_win_shade_item_in_both_menus(win, MENUTREE_TABS_NEXT_TAB,
             shade);
     multi_win_shade_item_in_both_menus(win, MENUTREE_TABS_MOVE_TAB_RIGHT,
@@ -663,11 +665,11 @@ static void renumber_tabs(MultiWin *win)
 {
     GList *link;
     int n = 0;
-    
+
     for (link = win->tabs; link; link = g_list_next(link))
     {
         MultiTab *tab = link->data;
-        
+
         multi_tab_set_full_window_title(tab, tab->window_title_template,
                 tab->window_title);
         ++n;
@@ -722,7 +724,7 @@ void multi_tab_move_to_new_window(MultiWin *win, MultiTab *tab, int position)
 {
     MultiWin *old_win = tab->parent;
     gboolean old_win_destroyed;
-    
+
     multi_win_set_always_show_tabs(win, old_win->always_show_tabs);
     multi_win_set_show_menu_bar(win, old_win->show_menu_bar);
     if (multi_win_is_fullscreen(old_win))
@@ -776,7 +778,7 @@ static void multi_tab_pack_for_single(MultiTab *tab, GtkContainer *nb)
 static void multi_win_pack_for_single_tab(MultiWin *win)
 {
     MultiTab *tab = win->tabs->data;
-    
+
 #if !GTK_CHECK_VERSION(3, 0, 0)
     g_object_set(G_OBJECT(win->notebook), "homogeneous", FALSE, NULL);
 #endif
@@ -795,14 +797,14 @@ static void multi_win_pack_for_multiple_tabs(MultiWin *win)
 {
     GList *link;
     GtkContainer *nb = GTK_CONTAINER(win->notebook);
-    
+
 #if !GTK_CHECK_VERSION(3, 0, 0)
     g_object_set(G_OBJECT(win->notebook), "homogeneous", TRUE, NULL);
 #endif
     for (link = win->tabs; link; link = g_list_next(link))
     {
         MultiTab *tab = link->data;
-        
+
         multi_tab_pack_for_multiple(tab, nb);
     }
 }
@@ -813,7 +815,7 @@ static void multi_win_set_full_title(MultiWin *win,
     if (win->gtkwin)
     {
         char *title0 = make_title(template, title);
-        
+
         gtk_window_set_title(GTK_WINDOW(win->gtkwin), title0);
         g_free(title0);
     }
@@ -851,7 +853,7 @@ void multi_win_select_tab(MultiWin * win, MultiTab * tab)
     if (tab)
     {
         char *title = multi_tab_get_full_window_title(tab);
-        
+
         if (tab->label)
             multitab_label_cancel_attention(MULTITAB_LABEL(tab->label));
         multi_tab_set_status_stock(tab, NULL);
@@ -946,6 +948,7 @@ void multi_win_set_fullscreen(MultiWin *win, gboolean fullscreen)
 static gboolean multi_win_state_event_handler(GtkWidget *widget,
         GdkEventWindowState *event, MultiWin *win)
 {
+    (void) widget;
     if (event->changed_mask & GDK_WINDOW_STATE_FULLSCREEN)
     {
         win->fullscreen =
@@ -993,6 +996,8 @@ multi_win_page_switched(GtkNotebook * notebook, GtkWidget *page,
 {
     MultiTab *tab = multi_tab_get_from_widget(
             gtk_notebook_get_nth_page(GTK_NOTEBOOK(win->notebook), page_num));
+    (void) notebook;
+    (void) page;
 
     if (win->current_tab == tab)
         return;
@@ -1003,7 +1008,8 @@ static void tab_set_name_from_clipboard_callback(GtkClipboard *clp,
         const gchar *text, gpointer data)
 {
     MultiTab *tab = (MultiTab *) data;
-    
+    (void) clp;
+
     if (text == NULL || tab == NULL)
         return;
     multi_tab_set_window_title_template(tab, text);
@@ -1012,6 +1018,8 @@ static void tab_set_name_from_clipboard_callback(GtkClipboard *clp,
 static gboolean multi_win_focus_in(GtkWidget *widget, GdkEventFocus *event,
         MultiWin *win)
 {
+    (void) widget;
+    (void) event;
     multi_win_all = g_list_remove(multi_win_all, win);
     multi_win_all = g_list_prepend(multi_win_all, win);
     gtk_window_set_urgency_hint(GTK_WINDOW(win->gtkwin), FALSE);
@@ -1021,7 +1029,7 @@ static gboolean multi_win_focus_in(GtkWidget *widget, GdkEventFocus *event,
 static void popup_tabs_menu(MultiWin *win, GdkEventButton *event)
 {
     GtkMenuItem *mparent;
-    
+
     g_return_if_fail(win->popup_menu != NULL);
     mparent = GTK_MENU_ITEM(
             menutree_get_widget_for_id(win->popup_menu, MENUTREE_TABS));
@@ -1032,9 +1040,10 @@ static void popup_tabs_menu(MultiWin *win, GdkEventButton *event)
 static gboolean tab_clicked_handler(GtkWidget *widget,
         GdkEventButton *event, MultiTab *tab)
 {
+    (void) widget;
     if (event->type != GDK_BUTTON_PRESS)
         return FALSE;
-    
+
     switch (event->button)
     {
         case 2:
@@ -1062,10 +1071,50 @@ static gboolean tab_clicked_handler(GtkWidget *widget,
     return FALSE;
 }
 
+/* Creates the label widget for a tab. tab->label is the GtkLabel containing
+ * the text; the return value is the top-level container. */
+static GtkWidget *make_tab_label(MultiTab *tab, GtkPositionType tab_pos)
+{
+    (void) tab_pos;
+    tab->label = multitab_label_new(tab->parent->notebook, NULL,
+            &tab->parent->best_tab_width);
+    multi_tab_set_full_window_title(tab, tab->window_title_template,
+            tab->window_title);
+#if GTK_CHECK_VERSION(3, 0, 0)
+    /* GtkBox will be replaced by GtkGrid one day, but at the moment it
+     * doesn't work properly because GtkNotebook has no homogeneous option
+     * and GtkGrid doesn't have pack_end to right-align close button.
+     */
+     /*
+    tab->label_box = gtk_grid_new();
+    gtk_orientable_set_orientation(GTK_ORIENTABLE(tab->label_box),
+            GTK_ORIENTATION_HORIZONTAL);
+    */
+    tab->label_box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4);
+    gtk_box_set_homogeneous(GTK_BOX(tab->label_box), FALSE);
+#else
+    tab->label_box = gtk_hbox_new(FALSE, 4);
+#endif
+    gtk_box_pack_start(GTK_BOX(tab->label_box), tab->label, TRUE, TRUE, 0);
+    g_signal_connect(tab->label, "button-press-event",
+            G_CALLBACK(tab_clicked_handler), tab);
+    if (multi_tab_get_show_close_button(tab->user_data))
+    {
+        multi_tab_add_close_button(tab);
+    }
+    gtk_notebook_set_tab_label(GTK_NOTEBOOK(tab->parent->notebook),
+        tab->widget, tab->label_box);
+    gtk_widget_show_all(tab->label_box);
+
+    return tab->label_box;
+}
+
 static void page_reordered_callback(GtkNotebook *notebook, GtkWidget *child,
         guint page_num, MultiWin *win)
 {
     MultiTab *tab = multi_tab_get_from_widget(child);
+    (void) notebook;
+    (void) win;
 
     g_return_if_fail(tab);
     multi_tab_move_to_position(tab, page_num, FALSE);
@@ -1077,55 +1126,66 @@ static void page_added_callback(GtkNotebook *notebook, GtkWidget *child,
     MultiTab *tab = multi_tab_get_from_widget(child);
     gboolean old_win_destroyed;
     MultiWin *old_win = tab->parent;
+    gboolean vert_tabs;
 
     g_return_if_fail(tab);
-    if (!win->ignore_tabs_moving)
+    if (win->ignore_tabs_moving)
+        return;
+    if (tab->parent == win)
     {
-        if (tab->parent == win)
+        /* Dragged in from same window so do nothing */
+        return;
+    }
+    vert_tabs = win->tab_pos == GTK_POS_LEFT || win->tab_pos == GTK_POS_RIGHT;
+    if (tab->parent)
+    {
+        old_win_destroyed = multi_tab_remove_from_parent(tab, TRUE);
+    }
+    else
+    {
+        old_win_destroyed = TRUE;
+        g_warning("Page added had no previous parent");
+    }
+    multi_win_add_tab(win, tab, page_num, TRUE);
+    multi_tab_to_new_window_handler(win, tab,
+            old_win_destroyed ? NULL : old_win);
+    if (vert_tabs)
+    {
+        multi_tab_pack_for_horizontal(tab, GTK_CONTAINER(notebook));
+    }
+    else
+    {
+        if (win->ntabs == 1)
         {
-            /* Dragged in from same window so do nothing */
-            return;
+            multi_win_pack_for_single_tab(win);
+            gtk_widget_queue_resize(win->notebook);
+            /* multi_tab_set_single_size(tab); */
+            multi_win_show(win);
         }
-        if (tab->parent)
+        else if (win->ntabs == 2)
         {
-            old_win_destroyed = multi_tab_remove_from_parent(tab, TRUE);
+            multi_win_pack_for_multiple_tabs(win);
         }
         else
         {
-            old_win_destroyed = TRUE;
-            g_warning("Page added had no previous parent");
+            multi_tab_pack_for_multiple(tab, GTK_CONTAINER(notebook));
         }
-        multi_win_add_tab(win, tab, page_num, TRUE);
-        multi_tab_to_new_window_handler(win, tab,
-                old_win_destroyed ? NULL : old_win);
-        if (win->tab_pos == GTK_POS_LEFT || win->tab_pos == GTK_POS_RIGHT)
+#if GTK_CHECK_VERSION(3, 0, 0)
+        if (old_win_destroyed && !vert_tabs)
         {
-            multi_tab_pack_for_horizontal(tab, GTK_CONTAINER(notebook));
-        }
-        else
-        {
-            if (win->ntabs == 1)
-            {
-                multi_win_pack_for_single_tab(win);
-                gtk_widget_queue_resize(win->notebook);
-                /* multi_tab_set_single_size(tab); */
-                multi_win_show(win);
-            }
-            else if (win->ntabs == 2)
-            {
-                multi_win_pack_for_multiple_tabs(win);
-            }
-            else
-            {
-                multi_tab_pack_for_multiple(tab, GTK_CONTAINER(notebook));
-            }
+            /* The dragged tab's label will now be stupidly small,
+             * but there's absolutely nothing we can do about it.
+             */
         }
+#endif
     }
 }
 
 static GtkNotebook *multi_win_notebook_creation_hook(GtkNotebook *source,
         GtkWidget *page, gint x, gint y, gpointer data)
 {
+    (void) source;
+    (void) data;
     MultiTab *tab = multi_tab_get_from_widget(page);
     MultiWin *win;
 
@@ -1155,7 +1215,8 @@ MultiWin *multi_win_new_for_tab(const char *display_name, int x, int y,
     win = multi_win_new_blank(display_name,
                 multi_win_get_shortcut_scheme(win), win->zoom_index,
                 disable_menu_shortcuts, disable_tab_shortcuts,
-                win->tab_pos, win->always_show_tabs);
+                win->tab_pos, win->always_show_tabs,
+                win->show_add_tab_button);
     multi_win_set_show_menu_bar(win, show_menubar);
     multi_win_set_geometry_hints_for_tab(win, tab);
     multi_win_set_title_template(win, title_template);
@@ -1169,6 +1230,7 @@ MultiWin *multi_win_new_for_tab(const char *display_name, int x, int y,
 
 static void multi_win_close_tab_clicked(GtkWidget *widget, MultiTab *tab)
 {
+    (void) widget;
     if (!multi_win_delete_handler ||
             !multi_win_delete_handler(tab->parent->gtkwin, NULL,
                     tab->user_data))
@@ -1191,7 +1253,7 @@ MultiWin *multi_win_clone(MultiWin *old,
     sprintf(geom, "%dx%d", width, height);
     return multi_win_new_with_geom(old->display_name, old->shortcuts,
             old->zoom_index, user_data_template, geom,
-            num_tabs, tab_pos, always_show_tabs);
+            num_tabs, tab_pos, always_show_tabs, old->show_add_tab_button);
 }
 
 static void multi_win_new_window_action(MultiWin * win)
@@ -1211,7 +1273,7 @@ static void multi_win_new_tab_action(MultiWin * win)
 static void multi_win_detach_tab_action(MultiWin * win)
 {
     MultiTab *tab = win->current_tab;
-    
+
     win = multi_win_new_for_tab(win->display_name, -1, -1, tab);
     multi_tab_move_to_new_window(win, tab, -1);
     gtk_widget_show_all(win->gtkwin);
@@ -1225,7 +1287,7 @@ static void multi_win_close_tab_action(MultiWin * win)
 static void multi_win_close_other_tabs_action(MultiWin * win)
 {
     GList *link;
-    
+
     while ((link = win->tabs) != NULL)
     {
         if (link->data == win->current_tab)
@@ -1455,7 +1517,7 @@ static void multi_win_vscroll_by(MultiWin *win, double multiplier,
     GtkAdjustment *adj;
     double newval;
     double bottom;
-    
+
     adj = win->current_tab->adjustment;
     if (!adj)
         return;
@@ -1470,7 +1532,7 @@ static void multi_win_vscroll_by(MultiWin *win, double multiplier,
     else
     {
         double step;
-        
+
         if (step_type == MULTI_TAB_SCROLL_STEP_PAGE)
             step = gtk_adjustment_get_page_increment(adj);
         else
@@ -1505,6 +1567,18 @@ static void multi_win_scroll_page_down_action(MultiWin *win)
     multi_win_vscroll_by(win, 1, MULTI_TAB_SCROLL_STEP_PAGE);
 }
 
+static void multi_win_scroll_half_page_up_action(MultiWin *win)
+{
+    g_return_if_fail(win->current_tab != NULL);
+    multi_win_vscroll_by(win, -0.5, MULTI_TAB_SCROLL_STEP_PAGE);
+}
+
+static void multi_win_scroll_half_page_down_action(MultiWin *win)
+{
+    g_return_if_fail(win->current_tab != NULL);
+    multi_win_vscroll_by(win, 0.5, MULTI_TAB_SCROLL_STEP_PAGE);
+}
+
 static void multi_win_scroll_to_top_action(MultiWin *win)
 {
     g_return_if_fail(win->current_tab != NULL);
@@ -1521,17 +1595,17 @@ static void multi_win_move_tab_by_one(MultiWin *win, int dir)
 {
     MultiTab *tab = win->current_tab;
     int pos;
-    
+
     g_return_if_fail(win->current_tab);
     pos = multi_tab_get_page_num(tab) + dir;
     if (win->wrap_switch_tab)
     {
         if (pos < 0)
             pos = win->ntabs - 1;
-        else if (pos >= win->ntabs)
+        else if (pos >= (int) win->ntabs)
             pos = 0;
     }
-    g_return_if_fail(pos >= 0 && pos < win->ntabs);
+    g_return_if_fail(pos >= 0 && pos < (int) win->ntabs);
     multi_tab_move_to_position(tab, pos, TRUE);
 }
 
@@ -1615,6 +1689,12 @@ static void multi_win_connect_actions(MultiWin * win)
         G_CALLBACK(multi_win_scroll_page_up_action), win, NULL, NULL, NULL);
     multi_win_menu_connect_swapped(win, MENUTREE_VIEW_SCROLL_PAGE_DOWN,
         G_CALLBACK(multi_win_scroll_page_down_action), win, NULL, NULL, NULL);
+    multi_win_menu_connect_swapped(win, MENUTREE_VIEW_SCROLL_HALF_PAGE_UP,
+        G_CALLBACK(multi_win_scroll_half_page_up_action), win,
+        NULL, NULL, NULL);
+    multi_win_menu_connect_swapped(win, MENUTREE_VIEW_SCROLL_HALF_PAGE_DOWN,
+        G_CALLBACK(multi_win_scroll_half_page_down_action), win,
+        NULL, NULL, NULL);
     multi_win_menu_connect_swapped(win, MENUTREE_VIEW_SCROLL_TO_TOP,
         G_CALLBACK(multi_win_scroll_to_top_action), win, NULL, NULL, NULL);
     multi_win_menu_connect_swapped(win, MENUTREE_VIEW_SCROLL_TO_BOTTOM,
@@ -1629,7 +1709,7 @@ static void multi_win_set_show_tabs_menu_items(MultiWin *win, gboolean active)
 
 static void multi_win_show_tabs(MultiWin * win)
 {
-    if (win->tab_pos != -1)
+    if ((int) win->tab_pos != -1)
     {
         gtk_notebook_set_show_tabs(GTK_NOTEBOOK(win->notebook), TRUE);
         gtk_notebook_set_show_border(GTK_NOTEBOOK(win->notebook), TRUE);
@@ -1688,13 +1768,13 @@ void multi_win_set_colormap(MultiWin *win)
         win->composite = FALSE;
     }
 }
-#endif
+#endif  /* !GTK3 */
 
 static void multi_win_composited_changed(GtkWidget *widget, MultiWin *win)
 {
     gboolean composited =
             gdk_screen_is_composited(gtk_widget_get_screen(widget));
-    
+
     if (composited != win->composite)
     {
         if (gtk_widget_get_realized(win->gtkwin))
@@ -1706,21 +1786,21 @@ static void multi_win_composited_changed(GtkWidget *widget, MultiWin *win)
             gboolean was_minimized;
             int x, y;
             GdkWindow *dwin = gtk_widget_get_window(win->gtkwin);
-            
+
             user_time = gdk_x11_display_get_user_time(
                     gtk_widget_get_display (widget));
-            
+
             /* If compositing changed, re-realize the window. Bug #563561 */
-            
+
             gtk_window_get_position(GTK_WINDOW(win->gtkwin), &x, &y);
             was_minimized = x11support_window_is_minimized(dwin);
             have_desktop = x11support_get_wm_desktop(dwin,
                     &desktop);
             gtk_widget_hide (widget);
             gtk_widget_unrealize (widget);
-            
+
             multi_win_set_colormap(win);
-            
+
             gtk_window_move(GTK_WINDOW(win->gtkwin), x, y);
             gtk_widget_realize(win->gtkwin);
             dwin = gtk_widget_get_window(win->gtkwin);
@@ -1744,6 +1824,7 @@ static void multi_win_composited_changed(GtkWidget *widget, MultiWin *win)
 static gboolean multi_win_map_event_handler(GtkWidget *widget,
         GdkEvent *event, MultiWin *win)
 {
+    (void) event;
     if (win->clear_demands_attention)
     {
         x11support_clear_demands_attention(gtk_widget_get_window(widget));
@@ -1754,10 +1835,24 @@ static gboolean multi_win_map_event_handler(GtkWidget *widget,
 
 #endif /* HAVE_COMPOSITE */
 
+/* Close window explicitly instead of allowing system to close it by returning
+ * FALSE from delete-event handler. This cures
+ * https://sourceforge.net/p/roxterm/bugs/89/
+ * but I don't really know why.
+ */
+static gboolean multi_win_delete_event_cb(GtkWidget *widget, GdkEvent *event,
+        MultiWin *win)
+{
+    if (!multi_win_delete_handler(widget, event, win))
+        multi_win_destructor(win, TRUE);
+    return TRUE;
+}
+
 MultiWin *multi_win_new_blank(const char *display_name, Options *shortcuts,
         int zoom_index,
         gboolean disable_menu_shortcuts, gboolean disable_tab_shortcuts,
-        GtkPositionType tab_pos, gboolean always_show_tabs)
+        GtkPositionType tab_pos, gboolean always_show_tabs,
+        gboolean add_tab_button)
 {
     MultiWin *win = g_new0(MultiWin, 1);
 #if !GTK_CHECK_VERSION(2, 24, 0)
@@ -1787,11 +1882,11 @@ MultiWin *multi_win_new_blank(const char *display_name, Options *shortcuts,
         win->zoom_index = zoom_index;
 
     win->gtkwin = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-    
+
     if (display_name)
     {
         GdkScreen *scr;
-        
+
         win->display_name = g_strdup(display_name);
         scr = display_get_screen_for_name(display_name);
         if (scr)
@@ -1819,9 +1914,9 @@ MultiWin *multi_win_new_blank(const char *display_name, Options *shortcuts,
     if (multi_win_delete_handler)
     {
         g_signal_connect(win->gtkwin, "delete-event",
-                G_CALLBACK(multi_win_delete_handler), win);
+                G_CALLBACK(multi_win_delete_event_cb), win);
     }
-    
+
 #if HAVE_COMPOSITE
     multi_win_set_colormap(win);
     g_signal_connect(win->gtkwin, "composited-changed",
@@ -1862,7 +1957,7 @@ MultiWin *multi_win_new_blank(const char *display_name, Options *shortcuts,
     menutree_connect_destroyed(win->menu_bar,
         G_CALLBACK(multi_win_menutree_deleted_handler), win);
     win->show_menu_bar = FALSE;
-    
+
     if (win->tab_pos == GTK_POS_LEFT || win->tab_pos == GTK_POS_RIGHT)
     {
         menutree_change_move_tab_labels(win->popup_menu);
@@ -1891,7 +1986,7 @@ MultiWin *multi_win_new_blank(const char *display_name, Options *shortcuts,
         g_object_set(notebook, "tab-vborder", 0, NULL);
     }
 #endif
-    if (tab_pos == -1)
+    if ((int) tab_pos == -1)
     {
         gtk_notebook_set_show_tabs(notebook, always_show_tabs);
         gtk_notebook_set_show_border(notebook, always_show_tabs);
@@ -1911,6 +2006,13 @@ MultiWin *multi_win_new_blank(const char *display_name, Options *shortcuts,
         multi_win_set_show_tabs_menu_items(win, FALSE);
         multi_win_hide_tabs(win);
     }
+    win->add_tab_button = multitab_close_button_new(GTK_STOCK_ADD);
+    g_signal_connect_swapped(win->add_tab_button, "clicked",
+            G_CALLBACK(multi_win_new_tab_action), win);
+    gtk_notebook_set_action_widget(notebook, win->add_tab_button, GTK_PACK_END);
+    if (add_tab_button)
+        gtk_widget_show_all(win->add_tab_button);
+    win->show_add_tab_button = add_tab_button;
     box_compat_packv(win->vbox, win->notebook, TRUE, 0);
     g_signal_connect(win->notebook, "switch-page",
         G_CALLBACK(multi_win_page_switched), win);
@@ -1942,18 +2044,18 @@ void multi_win_set_role_prefix(const char *role_prefix)
 MultiWin *multi_win_new_full(const char *display_name, Options *shortcuts,
         int zoom_index, gpointer user_data_template, const char *geom,
         MultiWinSizing sizing, int numtabs, GtkPositionType tab_pos,
-        gboolean always_show_tabs)
+        gboolean always_show_tabs, gboolean add_tab_button)
 {
-    gboolean disable_menu_shortcuts, disable_tab_shortcuts; 
+    gboolean disable_menu_shortcuts, disable_tab_shortcuts;
     MultiWin *win;
     MultiTab *tab;
     int n;
-    
+
     multi_win_get_disable_menu_shortcuts(user_data_template,
             &disable_menu_shortcuts, &disable_tab_shortcuts);
     win = multi_win_new_blank(display_name, shortcuts, zoom_index,
             disable_menu_shortcuts, disable_tab_shortcuts,
-            tab_pos, always_show_tabs);
+            tab_pos, always_show_tabs, add_tab_button);
     win->user_data_template = user_data_template;
     win->tab_pos = tab_pos;
     for (n = 0; n < numtabs; ++n)
@@ -2088,7 +2190,7 @@ void multi_tab_add_close_button(MultiTab *tab)
 
     if (tab->close_button)
         return;
-    
+
     win = tab->parent;
     tab->close_button = multitab_close_button_new(tab->status_stock);
     gtk_box_pack_start(GTK_BOX(tab->label_box), tab->close_button,
@@ -2107,7 +2209,7 @@ void multi_tab_remove_close_button(MultiTab *tab)
     if (tab->close_button)
     {
         MultiWin *win;
-    
+
         win = tab->parent;
         gtk_widget_destroy(tab->close_button);
         tab->close_button = NULL;
@@ -2138,50 +2240,13 @@ void multi_tab_set_middle_click_tab_action(MultiTab *tab, int action)
     tab->middle_click_action = action;
 }
 
-/* Creates the label widget for a tab. tab->label is the GtkLabel containing
- * the text; the return value is the top-level container. */
-static GtkWidget *make_tab_label(MultiTab *tab, GtkPositionType tab_pos)
-{
-    tab->label = multitab_label_new(tab->parent->notebook, NULL,
-            &tab->parent->best_tab_width);
-    multi_tab_set_full_window_title(tab, tab->window_title_template,
-            tab->window_title);
-#if GTK_CHECK_VERSION(3, 0, 0)
-    /* GtkBox will be replaced by GtkGrid one day, but at the moment it
-     * doesn't work properly because GtkNotebook has no homogeneous option
-     * and GtkGrid doesn't have pack_end to right-align close button.
-     */
-     /*
-    tab->label_box = gtk_grid_new();
-    gtk_orientable_set_orientation(GTK_ORIENTABLE(tab->label_box),
-            GTK_ORIENTATION_HORIZONTAL);
-    */
-    tab->label_box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4);
-    gtk_box_set_homogeneous(GTK_BOX(tab->label_box), FALSE);
-#else
-    tab->label_box = gtk_hbox_new(FALSE, 4);
-#endif
-    gtk_box_pack_start(GTK_BOX(tab->label_box), tab->label, TRUE, TRUE, 0);
-    g_signal_connect(tab->label, "button-press-event",
-            G_CALLBACK(tab_clicked_handler), tab);
-    if (multi_tab_get_show_close_button(tab->user_data))
-    {
-        multi_tab_add_close_button(tab);
-    }
-    gtk_notebook_set_tab_label(GTK_NOTEBOOK(tab->parent->notebook),
-        tab->widget, tab->label_box);
-    gtk_widget_show_all(tab->label_box);
-
-    return tab->label_box;
-}
-
 static void multi_tab_add_menutree_items(MultiWin * win, MultiTab * tab,
         int position)
 {
     char *title = multi_tab_get_full_window_title(tab);
     char *n_and_title = g_strdup_printf("%d. %s",
             multi_tab_get_page_num(tab), title);
-    
+
     tab->popup_menu_item = menutree_add_tab_at_position(win->popup_menu,
             n_and_title, position);
     tab->menu_bar_item = menutree_add_tab_at_position(win->menu_bar,
@@ -2235,7 +2300,7 @@ static void multi_win_add_tab_to_notebook(MultiWin * win, MultiTab * tab,
 }
 
 /* Keep track of a tab after it's been created; if notify_only is set, it's
- * assumed the tab has already been added by GTK and this is being called 
+ * assumed the tab has already been added by GTK and this is being called
  * just for notification */
 static void multi_win_add_tab(MultiWin * win, MultiTab * tab, int position,
         gboolean notify_only)
@@ -2286,7 +2351,7 @@ gboolean multi_win_is_fullscreen(MultiWin *win)
 gboolean multi_win_is_maximised(MultiWin *win)
 {
     GdkWindow *w = gtk_widget_get_window(win->gtkwin);
-    
+
     return (w && (gdk_window_get_state(w) & GDK_WINDOW_STATE_MAXIMIZED) != 0);
 }
 
@@ -2462,13 +2527,13 @@ gboolean multi_win_get_always_show_tabs(MultiWin *win)
 void multi_win_set_always_show_tabs(MultiWin *win, gboolean show)
 {
     gboolean old_show = win->always_show_tabs;
-    
+
     win->always_show_tabs = show;
     multi_win_set_show_tabs_menu_items(win, show);
     if (win->ntabs == 1 && old_show != show)
     {
         MultiTab *tab = win->current_tab;
-        
+
         g_return_if_fail(tab != NULL);
         if (show)
             multi_win_show_tabs(win);
@@ -2478,6 +2543,20 @@ void multi_win_set_always_show_tabs(MultiWin *win, gboolean show)
     }
 }
 
+gboolean multi_win_get_show_add_tab_button(MultiWin *win)
+{
+    return win->show_add_tab_button;
+}
+
+void multi_win_set_show_add_tab_button(MultiWin *win, gboolean show)
+{
+    win->show_add_tab_button = show;
+    if (show)
+        gtk_widget_show_all(win->add_tab_button);
+    else
+        gtk_widget_hide(win->add_tab_button);
+}
+
 void multi_win_set_title_template(MultiWin *win, const char *tt)
 {
     if (!validate_title_template(GTK_WINDOW(win->gtkwin), tt))
@@ -2526,7 +2605,7 @@ const char *multi_win_get_display_name(MultiWin *win)
 const char *multi_win_get_shortcuts_scheme_name(MultiWin *win)
 {
     char *slash = strrchr(win->shortcuts->name, G_DIR_SEPARATOR);
-    
+
     return slash ? slash + 1 : win->shortcuts->name;
 }
 
