@@ -24,6 +24,7 @@
 #include <string.h>
 
 #include "capplet.h"
+#include "colourgui.h"
 #include "configlet.h"
 #include "dlg.h"
 #include "dragrcv.h"
@@ -75,7 +76,7 @@ GtkWidget *profilegui_widget(ProfileGUI *pg, const char *name)
 {
     GtkWidget *widget =
             (GtkWidget *)gtk_builder_get_object(pg->capp.builder, name);
-    
+
     if (!widget)
         g_critical(_("Profile widget '%s' not found"), name);
     return widget;
@@ -179,6 +180,7 @@ void on_editable_changed(GtkEditable * editable, ProfileGUI *pg)
     else PG_IF_CHANGED(browser);
     else PG_IF_CHANGED(mailer);
     else PG_IF_CHANGED(dir_filer);
+    else PG_IF_CHANGED(filer);
     else PG_IF_CHANGED(command);
     else PG_IF_CHANGED(title_string);
     else PG_IF_CHANGED(ssh_address);
@@ -189,6 +191,7 @@ void on_editable_changed(GtkEditable * editable, ProfileGUI *pg)
 
 void on_ssh_entry_activate(GtkEntry * entry, ProfileGUI * pg)
 {
+    (void) entry;
     profilegui_check_ssh_entries_for_changes(pg);
 }
 
@@ -200,8 +203,9 @@ void on_ssh_host_changed(GtkWidget * widget, ProfileGUI *pg)
 
 void on_edit_ssh_clicked(GtkWidget * widget, ProfileGUI *pg)
 {
+    (void) widget;
     capplet_inc_windows();
-    
+
     capplet_set_text_entry(&pg->capp, "ssh_address", "localhost");
     capplet_set_text_entry(&pg->capp, "ssh_user", NULL);
     capplet_set_spin_button(&pg->capp, "ssh_port", 22);
@@ -214,6 +218,7 @@ void on_edit_ssh_clicked(GtkWidget * widget, ProfileGUI *pg)
 
 void on_Profile_Editor_destroy(GtkWidget * widget, ProfileGUI * pg)
 {
+    (void) widget;
     if (pg->ignore_destroy)
     {
         return;
@@ -227,11 +232,14 @@ void on_Profile_Editor_destroy(GtkWidget * widget, ProfileGUI * pg)
 
 void on_Profile_Editor_response(GtkWidget *widget, int response, ProfileGUI *pg)
 {
+    (void) widget;
+    (void) response;
     profilegui_delete(pg);
 }
 
 void on_Profile_Editor_close(GtkWidget * widget, ProfileGUI * pg)
 {
+    (void) widget;
     profilegui_delete(pg);
 }
 
@@ -243,12 +251,16 @@ void on_font_set(GtkFontButton * fontbutton, ProfileGUI * pg)
 
 void on_entry_activate(GtkEntry * entry, ProfileGUI * pg)
 {
+    (void) entry;
     profilegui_check_entries_for_changes(pg);
 }
 
 void on_profile_notebook_switch_page(GtkNotebook * notebook, GtkWidget *page,
         guint page_num, ProfileGUI * pg)
 {
+    (void) notebook;
+    (void) page;
+    (void) page_num;
     profilegui_check_entries_for_changes(pg);
 }
 
@@ -261,18 +273,20 @@ void on_bgtype_toggled(GtkToggleButton *button, ProfileGUI *pg)
 
 void on_command_toggled(GtkToggleButton *button, ProfileGUI *pg)
 {
+    (void) button;
     profilegui_set_command_shading(pg);
 }
 
 void on_close_buttons_toggled(GtkToggleButton *button, ProfileGUI *pg)
 {
+    (void) button;
     profilegui_set_close_buttons_shading(pg);
 }
 
 void on_cell_size_toggled(GtkToggleButton *button, ProfileGUI *pg)
 {
     gboolean state = gtk_toggle_button_get_active(button);
-    
+
     gtk_widget_set_sensitive(profilegui_widget(pg, "width"), state);
     gtk_widget_set_sensitive(profilegui_widget(pg, "height"), state);
 }
@@ -293,6 +307,7 @@ pg_load_preview(GtkFileChooser *chooser, GtkWidget *preview,
     int width, height;
     GdkPixbuf *pixbuf = NULL;
     GError *error = NULL;
+    (void) pg;
 
     if (filename && gdk_pixbuf_get_file_info(filename, &width, &height))
     {
@@ -313,7 +328,7 @@ pg_load_preview(GtkFileChooser *chooser, GtkWidget *preview,
                 PREVIEW_SIZE, PREVIEW_SIZE, TRUE, &error);
     }
     gtk_image_set_from_pixbuf(GTK_IMAGE(preview), pixbuf);
-    
+
     /* Let's not report file not found any more */
 #if 0
     if (!pixbuf && filename && filename[0] &&
@@ -346,7 +361,7 @@ static void on_update_preview(GtkFileChooser *chooser, ProfileGUI *pg)
     GtkWidget *preview = gtk_file_chooser_get_preview_widget(chooser);
     char *filename = gtk_file_chooser_get_preview_filename(chooser);
     GdkPixbuf *old_pixbuf = NULL;
-    
+
     if (gtk_image_get_storage_type(GTK_IMAGE(preview)) == GTK_IMAGE_PIXBUF)
         old_pixbuf = gtk_image_get_pixbuf(GTK_IMAGE(preview));
     if (old_pixbuf)
@@ -363,7 +378,7 @@ static void profilegui_setup_file_chooser(ProfileGUI *pg)
     GtkFileChooser *chooser =
             GTK_FILE_CHOOSER(gtk_builder_get_object(pg->capp.builder,
                     "background_img"));
-                    
+
     GtkFileFilter *img_filter = gtk_file_filter_new();
     GtkFileFilter *all_filter = gtk_file_filter_new();
 #ifdef MANAGE_PREVIEW
@@ -391,7 +406,7 @@ static void profilegui_fill_in_file_chooser(ProfileGUI *pg,
     GtkFileChooser *chooser =
             GTK_FILE_CHOOSER(gtk_builder_get_object(pg->capp.builder,
                     "background_img"));
-                    
+
 #ifdef MANAGE_PREVIEW
     GtkWidget *preview = gtk_file_chooser_get_preview_widget(chooser);
 #endif
@@ -425,6 +440,7 @@ void on_bgimg_chosen(GtkFileChooser *chooser, ProfileGUI *pg)
 
 void on_reset_compat_clicked(GtkButton *button, ProfileGUI *pg)
 {
+    (void) button;
     capplet_set_int(pg->capp.options, "backspace_binding",
             DEFAULT_BACKSPACE_BINDING);
     capplet_set_combo(&pg->capp, "backspace_binding",
@@ -435,6 +451,19 @@ void on_reset_compat_clicked(GtkButton *button, ProfileGUI *pg)
             DEFAULT_DELETE_BINDING);
 }
 
+void on_edit_colour_scheme_clicked(GtkButton *button, ProfileGUI *pg)
+{
+    (void) button;
+    GtkWidget *combo = capplet_lookup_combo(pg->capp.builder, "colour_scheme");
+    char *name;
+#ifdef HAVE_GTK_COMBO_BOX_TEXT_NEW
+    name = gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(combo));
+#else
+    name = gtk_combo_box_get_active_text(GTK_COMBO_BOX(combo));
+#endif
+    colourgui_open(name, gtk_widget_get_screen(combo));
+}
+
 static char *pg_get_dragged_in_filename(const char *text, gulong length)
 {
     char *first_file = first_file = strstr(text, "\r\n");
@@ -452,13 +481,14 @@ static gboolean bgimg_drag_data_received(GtkWidget *widget,
     ProfileGUI *pg = data;
     gboolean result = FALSE;
     char *filename = pg_get_dragged_in_filename(text, length);
+    (void) widget;
 
     if (gdk_pixbuf_get_file_info(filename, NULL, NULL))
     {
         GtkFileChooser *chooser =
                 GTK_FILE_CHOOSER(gtk_builder_get_object(pg->capp.builder,
                         "background_img"));
-                    
+
         gtk_file_chooser_set_filename(chooser, filename);
         /* Don't need to call capplet_set_string because we'll get a signal */
         gtk_toggle_button_set_active(
@@ -478,6 +508,192 @@ static void exit_action_changed(GtkComboBox *combo, ProfileGUI *pg)
 
 /***********************************************************/
 
+/* Hopefully all this combo support stuff can go away when everything
+ * supports GtkComboBoxText.
+ */
+
+inline static void profilegui_add_combo_item(GtkWidget *combo, const char *item)
+{
+#ifdef HAVE_GTK_COMBO_BOX_TEXT_NEW
+    gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(combo), item);
+#else
+    gtk_combo_box_append_text(GTK_COMBO_BOX(combo), item);
+#endif
+}
+
+static void profilegui_add_combo_items(GtkWidget *combo, ...)
+{
+    const char *item;
+    va_list ap;
+
+    va_start(ap, combo);
+    while ((item = va_arg(ap, const char *)) != NULL)
+    {
+        profilegui_add_combo_item(combo, item);
+    }
+    va_end(ap);
+}
+
+static void profilegui_add_combo_items_by_name(GtkWidget *combo,
+        const char *name)
+{
+    if (!strcmp(name, "exit_action"))
+    {
+        profilegui_add_combo_items(combo,
+            _("Exit terminal"),
+            _("Hold terminal open"),
+            _("Restart command"),
+            _("Ask user"),
+            NULL);
+    }
+    else if (!strcmp(name, "delete_binding") ||
+            !strcmp(name, "backspace_binding"))
+    {
+        profilegui_add_combo_items(combo,
+            _("Automatic choice"),
+            _("Control-H"),
+            _("ASCII DEL"),
+            _("Escape sequence"),
+            NULL);
+    }
+    else if (!strcmp(name, "tab_pos"))
+    {
+        profilegui_add_combo_items(combo,
+            _("Top"),
+            _("Bottom"),
+            _("Left"),
+            _("Right"),
+            _("Hidden"),
+            NULL);
+    }
+    else if (!strcmp(name, "colour_scheme"))
+    {
+        char **schemes = dynamic_options_list_sorted(
+                dynamic_options_get("Colours"));
+        int n;
+        profilegui_add_combo_item(combo, _("(Don't set)"));
+        if (!schemes[0] || strcmp(schemes[0], _("Default")))
+            profilegui_add_combo_item(combo, _("Default"));
+        for (n = 0; schemes[n]; ++n)
+            profilegui_add_combo_item(combo, schemes[n]);
+    }
+}
+
+static void on_colour_scheme_combo_changed(GtkComboBox *combo,
+        CappletData *capp)
+{
+    char *value;
+
+    if (capplet_ignore_changes)
+        return;
+
+    gboolean unset = gtk_combo_box_get_active(combo) == 0;
+    gtk_widget_set_sensitive(
+            GTK_WIDGET(gtk_builder_get_object(capp->builder,
+                    "edit_colour_scheme")), !unset);
+    if (unset)
+    {
+        value = NULL;
+    }
+    else
+    {
+#ifdef HAVE_GTK_COMBO_BOX_TEXT_NEW
+        value = gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(combo));
+#else
+        value = gtk_combo_box_get_active_text(combo);
+#endif
+    }
+    capplet_set_string(capp->options, "colour_scheme", value);
+    g_free(value);
+}
+
+static void profilegui_make_a_combo(ProfileGUI *pg, const char *name)
+{
+    char *box_name = g_strdup_printf("%s_hbox", name);
+    char *label_name = g_strdup_printf("%s_label", name);
+    GObject *box = gtk_builder_get_object(pg->capp.builder, box_name);
+    GObject *label = gtk_builder_get_object(pg->capp.builder, label_name);
+    GtkWidget *combo;
+
+    if (strcmp(name, "colour_scheme"))
+    {
+#ifdef HAVE_GTK_COMBO_BOX_TEXT_NEW
+        combo = gtk_combo_box_text_new();
+#else
+        combo = gtk_combo_box_new_text();
+#endif
+    }
+    else
+    {
+#ifdef HAVE_GTK_COMBO_BOX_TEXT_NEW
+        combo = gtk_combo_box_text_new_with_entry();
+#else
+        combo = gtk_combo_box_entry_new();
+#endif
+    }
+    gtk_label_set_mnemonic_widget(GTK_LABEL(label), combo);
+    profilegui_add_combo_items_by_name(combo, name);
+    gtk_buildable_set_name(GTK_BUILDABLE(combo), name);
+    gtk_buildable_add_child(GTK_BUILDABLE(box), pg->capp.builder,
+            G_OBJECT(combo), NULL);
+    if (strcmp(name, "colour_scheme"))
+    {
+        g_signal_connect(combo, "changed", G_CALLBACK(on_combo_changed), pg);
+    }
+    else
+    {
+        g_signal_connect(combo, "changed",
+                G_CALLBACK(on_colour_scheme_combo_changed), pg);
+    }
+    gtk_widget_show(combo);
+    g_free(label_name);
+    g_free(box_name);
+}
+
+static void profilegui_make_combos(ProfileGUI *pg)
+{
+    profilegui_make_a_combo(pg, "exit_action");
+    profilegui_make_a_combo(pg, "backspace_binding");
+    profilegui_make_a_combo(pg, "delete_binding");
+    profilegui_make_a_combo(pg, "tab_pos");
+    profilegui_make_a_combo(pg, "colour_scheme");
+}
+
+static void profilegui_set_colour_scheme_combo(CappletData *capp)
+{
+    GtkWidget *combo = capplet_lookup_combo(capp->builder, "colour_scheme");
+    char *value = options_lookup_string(capp->options, "colour_scheme");
+    int active = 0;
+    if (value && value[0])
+    {
+        GtkTreeModel *model = gtk_combo_box_get_model(GTK_COMBO_BOX(combo));
+        GtkTreeIter iter;
+        gboolean valid;
+        /* We skip the initial "(Don't set)" item */
+        active = 1;
+        for (valid = (gtk_tree_model_get_iter_first(model, &iter) &&
+                gtk_tree_model_iter_next(model, &iter));
+                valid;
+                valid = gtk_tree_model_iter_next(model, &iter), ++active)
+        {
+            char *item;
+            gtk_tree_model_get(model, &iter, 0, &item, -1);
+            if (!strcmp(item, value))
+            {
+                g_free(item);
+                break;
+            }
+            g_free(item);
+        }
+        if (!valid)
+            profilegui_add_combo_item(combo, value);
+    }
+    gtk_combo_box_set_active(GTK_COMBO_BOX(combo), active);
+    gtk_widget_set_sensitive(
+            GTK_WIDGET(gtk_builder_get_object(capp->builder,
+                    "edit_colour_scheme")), active != 0);
+    g_free(value);
+}
 
 static void profilegui_connect_handlers(ProfileGUI * pg)
 {
@@ -519,6 +735,7 @@ static void profilegui_fill_in_dialog(ProfileGUI * pg)
     capplet_set_boolean_toggle(&pg->capp, "tab_close_btn", TRUE);
     capplet_set_boolean_toggle(&pg->capp, "show_tab_status", FALSE);
     capplet_set_boolean_toggle(&pg->capp, "always_show_tabs", TRUE);
+    capplet_set_boolean_toggle(&pg->capp, "show_add_tab_btn", TRUE);
     capplet_set_boolean_toggle(&pg->capp, "new_tabs_adjacent", FALSE);
     capplet_set_boolean_toggle(&pg->capp, "show_tab_num", TRUE);
     capplet_set_radio(&pg->capp, "middle_click_tab", 0);
@@ -538,7 +755,7 @@ static void profilegui_fill_in_dialog(ProfileGUI * pg)
     capplet_set_boolean_toggle(&pg->capp, "file_as_dir", TRUE);
     capplet_set_boolean_toggle(&pg->capp, "match_plain_files", FALSE);
     capplet_set_spin_button(&pg->capp, "width", 80);
-    capplet_set_spin_button(&pg->capp, "height", 25);
+    capplet_set_spin_button(&pg->capp, "height", 24);
     capplet_set_radio(&pg->capp, "background_type", 0);
     on_cell_size_toggled(GTK_TOGGLE_BUTTON(profilegui_widget(pg, "cell_size")),
             pg);
@@ -574,6 +791,7 @@ static void profilegui_fill_in_dialog(ProfileGUI * pg)
     capplet_set_boolean_toggle(&pg->capp, "use_custom_command", FALSE);
     capplet_set_text_entry(&pg->capp, "command", NULL);
     capplet_set_combo(&pg->capp, "exit_action", 0);
+    profilegui_set_colour_scheme_combo(&pg->capp);
     capplet_set_spin_button_float(&pg->capp, "exit_pause");
     capplet_set_text_entry(&pg->capp, "title_string", "%s");
     capplet_set_text_entry(&pg->capp, "win_title", "%s");
@@ -589,7 +807,9 @@ static gboolean page_selected(GtkTreeSelection *selection,
 {
     GtkTreeIter iter;
     int page;
-    
+    (void) path_currently_selected;
+    (void) selection;
+
     if (!gtk_tree_model_get_iter(model, &iter, path))
         return FALSE;
     gtk_tree_model_get(model, &iter, 1, &page, -1);
@@ -606,12 +826,12 @@ static void profilegui_setup_list_store(ProfileGUI *pg)
             N_("Keyboard"), N_("Tabs")
     };
     GtkTreeIter iter;
-    int n;
+    guint n;
     GtkTreeViewColumn *column;
     GtkWidget *tvw;
     GtkTreeView *tv;
     GtkTreeSelection *sel;
-    
+
     pg->list_store = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_INT);
     for (n = 0; n < G_N_ELEMENTS(labels); ++n)
     {
@@ -634,93 +854,6 @@ static void profilegui_setup_list_store(ProfileGUI *pg)
     gtk_widget_show(tvw);
 }
 
-/* Hopefully all this combo support stuff can go away when everything
- * supports GtkComboBoxText.
- */
-
-static void profilegui_add_combo_items(GtkWidget *combo, ...)
-{
-    const char *item;
-    va_list ap;
-#ifdef HAVE_GTK_COMBO_BOX_TEXT_NEW
-    GtkComboBoxText *cast_combo = GTK_COMBO_BOX_TEXT(combo);
-#else
-    GtkComboBox *cast_combo = GTK_COMBO_BOX(combo);
-#endif
-    
-    va_start(ap, combo);
-    while ((item = va_arg(ap, const char *)) != NULL)
-    {
-#ifdef HAVE_GTK_COMBO_BOX_TEXT_NEW
-        gtk_combo_box_text_append_text(cast_combo, item);
-#else
-        gtk_combo_box_append_text(cast_combo, item);
-#endif
-    }
-    va_end(ap);
-}
-
-static void profilegui_add_combo_items_by_name(GtkWidget *combo,
-        const char *name)
-{
-    if (!strcmp(name, "exit_action"))
-    {
-        profilegui_add_combo_items(combo,
-            _("Exit terminal"),
-            _("Hold terminal open"),
-            _("Restart command"),
-            _("Ask user"),
-            NULL);
-    }
-    else if (!strcmp(name, "delete_binding") ||
-            !strcmp(name, "backspace_binding"))
-    {
-        profilegui_add_combo_items(combo,
-            _("Automatic choice"),
-            _("Control-H"),
-            _("ASCII DEL"),
-            _("Escape sequence"),
-            NULL);
-    }
-    else if (!strcmp(name, "tab_pos"))
-    {
-        profilegui_add_combo_items(combo,
-            _("Top"),
-            _("Bottom"),
-            _("Left"),
-            _("Right"),
-            _("Hidden"),
-            NULL);
-    }
-}
-
-static void profilegui_make_a_combo(ProfileGUI *pg, const char *name)
-{
-    char *box_name = g_strdup_printf("%s_hbox", name);
-    GObject *box = gtk_builder_get_object(pg->capp.builder, box_name);
-    GtkWidget *combo;
-    
-#ifdef HAVE_GTK_COMBO_BOX_TEXT_NEW
-    combo = gtk_combo_box_text_new();
-#else
-    combo = gtk_combo_box_new_text();
-#endif
-    profilegui_add_combo_items_by_name(combo, name);
-    gtk_buildable_set_name(GTK_BUILDABLE(combo), name);
-    gtk_buildable_add_child(GTK_BUILDABLE(box), pg->capp.builder,
-            G_OBJECT(combo), NULL);
-    g_signal_connect(combo, "changed", G_CALLBACK(on_combo_changed), pg);
-    gtk_widget_show(combo);
-}
-
-static void profilegui_make_combos(ProfileGUI *pg)
-{
-    profilegui_make_a_combo(pg, "exit_action");
-    profilegui_make_a_combo(pg, "backspace_binding");
-    profilegui_make_a_combo(pg, "delete_binding");
-    profilegui_make_a_combo(pg, "tab_pos");
-}
-
 /* Loads a Profile and creates a working dialog box for it */
 ProfileGUI *profilegui_open(const char *profile_name, GdkScreen *scrn)
 {
@@ -734,7 +867,7 @@ ProfileGUI *profilegui_open(const char *profile_name, GdkScreen *scrn)
             NULL };
     static const char *obj_names[] = {
             "Profile_Editor", "ssh_dialog",
-            "general_entries_size_group", "general_entry_labels_size_group", 
+            "general_entries_size_group", "general_entry_labels_size_group",
             NULL };
     GError *error = NULL;
 
@@ -751,7 +884,7 @@ ProfileGUI *profilegui_open(const char *profile_name, GdkScreen *scrn)
         gtk_window_present(GTK_WINDOW(pg->dialog));
         return pg;
     }
-    
+
     if (!profiles)
         profiles = dynamic_options_get("Profiles");
 
@@ -761,25 +894,25 @@ ProfileGUI *profilegui_open(const char *profile_name, GdkScreen *scrn)
         "roxterm profile");
 
     pg->capp.builder = gtk_builder_new();
-    if (!gtk_builder_add_objects_from_file(pg->capp.builder, 
+    if (!gtk_builder_add_objects_from_file(pg->capp.builder,
             capplet_get_ui_filename(), (char **) adj_names, &error) ||
-            !gtk_builder_add_objects_from_file(pg->capp.builder, 
+            !gtk_builder_add_objects_from_file(pg->capp.builder,
             capplet_get_ui_filename(), (char **) obj_names, &error))
     {
         g_error(_("Unable to load GTK UI definitions: %s"), error->message);
     }
     pg->dialog = profilegui_widget(pg, "Profile_Editor");
     pg->ssh_dialog = profilegui_widget(pg, "ssh_dialog");
-    
+
     if (scrn)
         gtk_window_set_screen(GTK_WINDOW(pg->dialog), scrn);
 
     title = g_strdup_printf(_("ROXTerm Profile \"%s\""), profile_name);
     gtk_window_set_title(GTK_WINDOW(pg->dialog), title);
     g_free(title);
-    
+
     profilegui_make_combos(pg);
-    
+
     profilegui_setup_list_store(pg);
 
     g_hash_table_insert(profilegui_being_edited, g_strdup(profile_name), pg);
